'''

This crib sheet shows how to load and plot EIS data

'''

from pyspedas import mms_load_hpca, tnames
from pytplot import tplot

# load ExTOF data
mms_load_eis(trange=['2015-10-16', '2015-10-17'], datatype='extof')

# plot the H+ flux
tplot(tnames('*_extof_proton_flux_omni'))

# load PHxTOF data
mms_load_eis(trange=['2015-10-16', '2015-10-17'], datatype='phxtof')

# plot the H+ flux
tplot(tnames('*_phxtof_proton_flux_omni'))



#!/usr/bin/python
# -*- coding: utf-8 -*-

from dateutil.parser import parse
import os

import pytplot

from .download_files_utilities import *
from .orbit_time import orbit_time


def maven_filenames(filenames=None,
                    instruments=None,
                    level='l2',
                    insitu=True,
                    iuvs=False,
                    start_date='2014-01-01',
                    end_date='2020-01-01',
                    update_prefs=False,
                    only_update_prefs=False,
                    local_dir=None):
    """
    This function identifies which MAVEN data to download.
    """

    # Check for orbit num rather than time string
    if isinstance(start_date, int) and isinstance(end_date, int):
        start_date, end_date = orbit_time(start_date, end_date)
        start_date = parse(start_date)
        end_date = parse(end_date)
        start_date = start_date.replace(hour=0, minute=0, second=0)
        end_date = end_date.replace(day=end_date.day+1, hour=0, minute=0, second=0)
        start_date = start_date.strftime('%Y-%m-%d')
        end_date = end_date.strftime('%Y-%m-%d')
        
    if update_prefs or only_update_prefs:
        set_new_data_root_dir()
        if only_update_prefs:
            return
    
    public = get_access()
    if not public:
        get_uname_and_password()

    if filenames is None:
        if insitu and iuvs:
            print("Can't request both INSITU and IUVS in one query.")
            return
        if not insitu and not iuvs:
            print("If not specifying filename(s) to download, Must specify either insitu=True or iuvs=True.")
            return
        
    if instruments is None:
        instruments = ['kp']
        if insitu:
            level = 'insitu'
        if iuvs:
            level = 'iuvs'

    # Set data download location
    if local_dir is None:
        mvn_root_data_dir = get_root_data_dir()
    else:
        mvn_root_data_dir = local_dir

    # Keep track of files to download
    maven_files = {}
    for instrument in instruments:
        # Build the query to the website
        query_args = []
        query_args.append("instrument=" + instrument)
        query_args.append("level=" + level)
        if filenames is not None:
            query_args.append("file=" + filenames)
        query_args.append("start_date=" + start_date)
        query_args.append("end_date=" + end_date)
        if level == 'iuvs':
            query_args.append("file_extension=tab")

        data_dir = os.path.join(mvn_root_data_dir, 'maven', 'data', 'sci', instrument, level)
        
        query = '&'.join(query_args)
        
        s = get_filenames(query, public)

        if not s:
            print("No files found for {}.".format(instrument))
            maven_files[instrument] = []
            continue

        s = s.split(',')

        maven_files[instrument] = [s, data_dir, public]

    return maven_files


def load_data(filenames=None,
              instruments=None,
              level='l2',
              insitu=True,
              iuvs=False,
              start_date='2014-01-01',
              end_date='2020-01-01',
              update_prefs=False,
              only_update_prefs=False,
              local_dir=None,
              list_files=False,
              new_files=False,
              exclude_orbit_file=False,
              download_only=False,
              varformat=None,
              prefix='',
              suffix='',
              get_support_data=False):
    """
    This function downloads MAVEN data loads it into tplot variables, if applicable.
    """

    # 1. Download files

    maven_files = maven_filenames(filenames, instruments, level, insitu, iuvs, start_date, end_date, update_prefs,
                                  only_update_prefs, local_dir)

    # Keep track of what files are downloaded
    downloaded_files = []

    for instr in maven_files.keys():
        if maven_files[instr]:
            s = maven_files[instr][0]
            data_dir = maven_files[instr][1]
            public = maven_files[instr][2]
            if list_files:
                for f in s:
                    print(f)
                return

            if new_files:
                s = get_new_files(s, data_dir, instr, level)

            print("Your request will download a total of: "+str(len(s))+" files for instrument "+str(instr))
            print('Would you like to proceed with the download? ')
            valid_response = False
            cancel = False
            while not valid_response:
                response = (input('(y/n) >  '))
                if response == 'y' or response == 'Y':
                    valid_response = True
                    cancel = False
                elif response == 'n' or response == 'N':
                    print('Cancelled download. Returning...')
                    valid_response = True
                    cancel = True
                else:
                    print('Invalid input.  Please answer with y or n.')

            if cancel:
                continue

            if not exclude_orbit_file:
                print("Before downloading data files, checking for updated orbit # file from naif.jpl.nasa.gov")
                print("")
                get_orbit_files()

            i = 0
            display_progress(i, len(s))
            for f in s:
                i = i+1
                full_path = create_dir_if_needed(f, data_dir, level)
                get_file_from_site(f, public, full_path)
                display_progress(i, len(s))

                downloaded_files.append(os.path.join(full_path, f))

    # 2. Load files into tplot

    if downloaded_files:
        # Flatten out downloaded files from list of lists of filenames
        if isinstance(downloaded_files[0], list):
            downloaded_files = [item for sublist in downloaded_files for item in sublist]

        # Only load in files into tplot if we actually downloaded CDF files
        cdf_files = [f for f in downloaded_files if '.cdf' in f]

        if not download_only:
            # Create tplot variables
            downloaded_tplot_vars = pytplot.cdf_to_tplot(cdf_files, varformat=varformat,
                                                         get_support_data=get_support_data, prefix=prefix,
                                                         suffix=suffix, merge=True)
            return downloaded_tplot_vars



'''
MMS Data in Python with pySPEDAS
Eric Grimes - egrimes@igpp.ucla.edu

*** Disclaimer: all of this is beta; please submit bug reports! ***
*** Load times can be slow for some MMS data - should be fixed soon! ***

Tentative agenda:
- Introduction
- Loading MMS Data in Python
- Plotting MMS Data in Python
- Analysis Tools
- Future Plans

'''

''' ==========================================================================

Introduction

pySPEDAS bleeding edge:
https://github.com/spedas/pyspedas

pyTplot:
https://github.com/MAVENSDC/PyTplot

MMS Datasets:
https://lasp.colorado.edu/mms/sdc/public/datasets/

========================================================================== ''' 

# installing: pip install pyspedas or download bleeding edge from Github
# configuration: edit mms_config.py
#                MMS_DATA_DIR environment variable
# PYTHONSTARTUP file: mms_python_startup.py - allows you to avoid having to manually import the load routines and basic tools

''' ==========================================================================

Loading MMS Data in Python

Load routines: 
    mms_load_xxx, where xxx is:
        MEC: Magnetic Ephemeris Coordinates
        FGM: Flux-gate Magnetometer
        SCM: Search-coil Magnetometer
        EDP: Electric field Double Probe (SDP+ADP)
        EDI: Electron Drift Instrument
        EIS: Energetic Ion Spectrometer 
        FEEPS: Fly's Eye Energetic Particle Sensor 
        HPCA: Hot Plasma Composition Analyzer
        FPI: Fast Plasma Investigation
        ASPOC: Active Spacecraft Potential Control

========================================================================== ''' 
from pyspedas.mms import mms_load_mec

# start by loading some ephemeris / coordinates data;
# the default trange is Oct 16, 2015, default probe is 1
# and the default data rate is 'srvy'
mms_load_mec(probe=4)

# find info on a load routine
help(mms_load_mec)

from pyspedas.mms import mms_load_fgm

# note that the keywords are the same as in IDL
mms_load_fgm(probe='4', data_rate='brst', trange=['2015-10-16/13:06', '2015-10-16/13:07'], time_clip=True)

# find which variables were loaded
from pytplot import tplot_names
tplot_names()

from pyspedas.mms import mms_load_edp

# load some burst mode electric field data
mms_load_edp(probe='4', data_rate='brst', trange=['2015-10-16/13:06', '2015-10-16/13:07'], time_clip=True)

from pyspedas import tnames

# the tnames function supports filtering with wild cards, e.g.,
# to find the E-field variables:
dce_vars = tnames('*_edp_dce_*')

# trange also accepts datetime objects
# note: be aware of potential time zone issues
from pyspedas.mms import mms_load_fpi
from datetime import datetime
from datetime import timezone as tz

start_time = datetime(year=2015, month=10, day=16, hour=13, minute=6, tzinfo=tz.utc)
end_time = datetime(year=2015, month=10, day=16, hour=13, minute=7, tzinfo=tz.utc)

mms_load_fpi(trange=[start_time, end_time], probe='4', datatype='des-moms', data_rate='brst')

# to return the actual data values, use get_data
from pytplot import get_data

times, fgm_data = get_data('mms4_fgm_b_gsm_brst_l2')

# times are unix time (seconds since 1 January 1970)
print(times[0])

# FGM data include the magnitude
fgm_data[0]

# you can convert the unix time to a string with time_string
from pyspedas import time_string

print(time_string(times[0]))

# and convert back to unix time using time_double
from pyspedas import time_double

print(time_double('2015-10-16 13:06:00.00451'))

# create new tplot variables with store_data
from pytplot import store_data

# save the B-field vector
store_data('b_vector', data={'x': times, 'y': fgm_data[:, 0:3]})
# save the B-field magnitude
store_data('b_mag', data={'x': times, 'y': fgm_data[:, 3]})

# the keywords are very flexible, e.g.,
from pyspedas.mms import mms_load_hpca, mms_load_eis, mms_load_feeps, mms_load_aspoc

# specify multiple probes as integers, and multiple datatypes
mms_load_hpca(probe=[1, 2, 4], data_rate='brst', datatype=['moments', 'ion'], trange=['2015-10-16/13:06', '2015-10-16/13:07'])


mms_load_eis(probe='4', data_rate='brst', datatype=['phxtof', 'extof'], trange=['2015-10-16/13:06', '2015-10-16/13:07'])
mms_load_feeps(get_support_data=True, probe=2, data_rate=['srvy', 'brst'], trange=['2015-10-16/13:06', '2015-10-16/13:07'])
mms_load_aspoc()

''' ==========================================================================

Plotting MMS Data in Python

========================================================================== ''' 

from pytplot import tplot

# like in IDL, pyTplot supports strings, lists of strings, as well as tplot variable #s
tplot('mms4_fgm_b_gsm_brst_l2')

tplot(['mms4_fgm_b_gsm_brst_l2', 'mms4_edp_dce_gse_brst_l2'])

# change the plot metadata
from pytplot import options
options('mms4_edp_dce_gse_brst_l2', 'color', ['b', 'g', 'r'])
options('mms4_edp_dce_gse_brst_l2', 'legend_names', ['Ex', 'Ey', 'Ez'])

tplot(['mms4_fgm_b_gsm_brst_l2', 'mms4_edp_dce_gse_brst_l2', 'mms4_des_energyspectr_omni_brst', 'mms4_des_pitchangdist_miden_brst'])

tplot('b_vector')
options('b_vector', 'ytitle', 'MMS4 FGM')
options('b_vector', 'color', ['b', 'g', 'r'])
options('b_vector', 'legend_names', ['Bx', 'By', 'Bz'])

tplot('b_vector')

# add vertical bars at certain times
from pytplot import timebar

timebar([time_double('2015-10-16/13:06:20'), time_double('2015-10-16/13:06:40')], varname='mms4_fgm_b_gsm_brst_l2')
tplot('mms4_fgm_b_gsm_brst_l2')

tplot(['mms4_des_numberdensity_brst', 'mms4_hpca_hplus_number_density'])

# set the y-axis to log scale
options('mms4_des_numberdensity_brst', 'ylog', True)
options('mms4_hpca_hplus_number_density', 'ylog', True)

tplot(['mms4_des_numberdensity_brst', 'mms4_hpca_hplus_number_density'])

''' ==========================================================================

Analysis Tools

========================================================================== ''' 

# subtract_average
from pyspedas import subtract_average

subtract_average('mms4_fgm_b_gsm_brst_l2')

tplot(['mms4_fgm_b_gsm_brst_l2', 'mms4_fgm_b_gsm_brst_l2-d'])

# subtract_median
from pyspedas import subtract_median

subtract_median('mms4_fgm_b_gsm_brst_l2')

tplot(['mms4_fgm_b_gsm_brst_l2', 'mms4_fgm_b_gsm_brst_l2-d', 'mms4_fgm_b_gsm_brst_l2-m'])

# time clip
from pyspedas import time_clip

time_clip(['mms4_fgm_b_gsm_brst_l2', 'mms4_fgm_b_gsm_brst_l2-d', 'mms4_fgm_b_gsm_brst_l2-m'], '2015-10-16/13:06:45', '2015-10-16/13:07', suffix='')

tplot(['mms4_fgm_b_gsm_brst_l2', 'mms4_fgm_b_gsm_brst_l2-d', 'mms4_fgm_b_gsm_brst_l2-m'])

# clip the data
from pyspedas import tclip

tclip(['mms4_fgm_b_gsm_brst_l2', 'mms4_fgm_b_gsm_brst_l2-d', 'mms4_fgm_b_gsm_brst_l2-m'], 0, 20, suffix='')

tplot(['mms4_fgm_b_gsm_brst_l2', 'mms4_fgm_b_gsm_brst_l2-d', 'mms4_fgm_b_gsm_brst_l2-m'])


''' ==========================================================================

Future Plans

- Speed up CDF load times 
- Searchable MMS events database 
- Corrected FEEPS omni-directional spectra 
- Omni-directional HPCA spectra (summed over anodes+spin averaged)
- Spin averaged EIS, FEEPS spectra 
- EIS, FEEPS pitch angle distributions 
- FPI lossy compression, error bars
- Read-only mirror support
- Tests, documentation, crib sheets

========================================================================== ''' 



'''
 PURPOSE:
       Apply flat field correction factors to FEEPS ion/electron data;
       correct factors are from the gain factor found in:
       
           FlatFieldResults_V3.xlsx
           
       from Drew Turner, 1/19/2017

 NOTES:
    
   From Drew Turner, 1/18/17:
       Here are the correction factors that we need to apply to the current 
       ION counts/rates/fluxes in the CDF files.  
       NOTE, THIS IS A DIFFERENT TYPE OF CORRECTION THAN THAT FOR THE ELECTRONS!  
       These shifts should be applied to the counts/rates/fluxes data EYE-BY-EYE on each spacecraft.  
       These are multiplication factors (i.e., Jnew = Jold * Gcorr). 
       For those equations, Jold is the original count/rate/flux array and
       Jnew is the corrected version of the arrays using the factors listed below.
       
        MMS1:
        Top6: Gcorr = 0.7
        Top7: Gcorr = 2.5
        Top8: Gcorr = 1.5
        Bot6: Gcorr = 0.9
        Bot7: Gcorr = 1.2
        Bot8: Gcorr = 1.0

        MMS2:
        Top6: Gcorr = 1.3
        Top7: BAD EYE
        Top8: Gcorr = 0.8
        Bot6: Gcorr = 1.4
        Bot7: BAD EYE
        Bot8: Gcorr = 1.5

        MMS3:
        Top6: Gcorr = 0.7
        Top7: Gcorr = 0.8
        Top8: Gcorr = 1.0
        Bot6: Gcorr = 0.9
        Bot7: Gcorr = 0.9
        Bot8: Gcorr = 1.3

        MMS4:
        Top6: Gcorr = 0.8
        Top7: BAD EYE
        Top8: Gcorr = 1.0
        Bot6: Gcorr = 0.8
        Bot7: Gcorr = 0.6
        Bot8: Gcorr = 0.9

'''

import pytplot
from pytplot import get_data, store_data
from pyspedas import tnames

def mms_feeps_flat_field_corrections(probes = ['1', '2', '3', '4'], data_rate = 'brst', suffix = ''):
    G_corr = {}
    G_corr['mms1-top6'] = 0.7
    G_corr['mms1-top7'] = 2.5
    G_corr['mms1-top8'] = 1.5
    G_corr['mms1-bot5'] = 1.2 # updated 1/24
    G_corr['mms1-bot6'] = 0.9
    G_corr['mms1-bot7'] = 2.2 # updated 1/24
    G_corr['mms1-bot8'] = 1.0

    G_corr['mms2-top4'] = 1.2 # added 1/24
    G_corr['mms2-top6'] = 1.3
    G_corr['mms2-top7'] = 0 # bad eye
    G_corr['mms2-top8'] = 0.8
    G_corr['mms2-bot6'] = 1.4
    G_corr['mms2-bot7'] = 0 # bad eye
    G_corr['mms2-bot8'] = 1.5

    G_corr['mms3-top6'] = 0.7
    G_corr['mms3-top7'] = 0.8
    G_corr['mms3-top8'] = 1.0
    G_corr['mms3-bot6'] = 0.9
    G_corr['mms3-bot7'] = 0.9
    G_corr['mms3-bot8'] = 1.3

    G_corr['mms4-top6'] = 0.8
    G_corr['mms4-top7'] = 0 # bad eye
    G_corr['mms4-top8'] = 1.0
    G_corr['mms4-bot6'] = 0.8
    G_corr['mms4-bot7'] = 0.6
    G_corr['mms4-bot8'] = 0.9
    G_corr['mms4-bot9'] = 1.5 # added 1/24

    #sensor_ids = ['6', '7', '8']
    sensor_ids = ['1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11', '12']
    sensor_types = ['top', 'bottom']
    levels = ['l2']
    species = ['ion', 'electron']

    for probe in probes:
        for sensor_type in sensor_types:
            for sensor_id in sensor_ids:
                if G_corr.get('mms'+probe+'-'+sensor_type[0:3]+sensor_id) is not None:
                    correction = G_corr['mms'+probe+'-'+sensor_type[0:3]+sensor_id]
                else:
                    correction = 1.0

                for level in levels:
                    for species_id in species:
                        if correction != 1.0:
                            cr_var = tnames('mms'+probe+'_epd_feeps_'+data_rate+'_'+level+'_'+species_id+'_'+sensor_type+'_count_rate_sensorid_'+sensor_id+suffix)
                            i_var = tnames('mms'+probe+'_epd_feeps_'+data_rate+'_'+level+'_'+species_id+'_'+sensor_type+'_intensity_sensorid_'+sensor_id+suffix)
                            c_var = tnames('mms'+probe+'_epd_feeps_'+data_rate+'_'+level+'_'+species_id+'_'+sensor_type+'_counts_sensorid_'+sensor_id+suffix)

                            if cr_var == []:
                                count_rate = None
                            else:
                                count_rate = get_data(cr_var[0])
                            if count_rate is not None:
                                cr_times, cr_data, cr_energies = count_rate
                                store_data(cr_var[0], data={'x': cr_times, 'y': cr_data*correction, 'v': cr_energies})

                            if i_var == []:
                                intensity = None
                            else:
                                intensity = get_data(i_var[0])
                            if intensity is not None:
                                i_times, i_data, i_energies = intensity
                                store_data(i_var[0], data={'x': i_times, 'y': i_data*correction, 'v': i_energies})

                            if c_var == []:
                                counts = None
                            else:
                                counts = get_data(c_var[0])
                            if counts is not None:
                                c_times, c_data, c_energies = counts
                                store_data(c_var[0], data={'x': c_times, 'y': c_data*correction, 'v': c_energies})



